@@ -34,18 +34,20 @@ GPL.
 // enigmailCommon.js: shared JS functions for Enigmail
 
 // This Enigmail version and compatible Enigmime version
-var gEnigmailVersion = "0.95.5";
-var gEnigmimeVersion = "0.95.5.0";
+var gEnigmailVersion = "1.1.2";
+var gEnigmimeVersion = "1.1.2";
 
 // Maximum size of message directly processed by Enigmail
 const ENIG_MSG_BUFFER_SIZE = 96000;
 const ENIG_MSG_HEADER_SIZE = 16000;
+const ENIG_UNLIMITED_BUFFER_SIZE = -1;
 
 const ENIG_KEY_BUFFER_SIZE = 64000;
 
 const ENIG_PROCESSINFO_CONTRACTID = "@mozilla.org/xpcom/process-info;1";
 const ENIG_PIPECONSOLE_CONTRACTID = "@mozilla.org/process/pipe-console;1";
 const ENIG_IPCBUFFER_CONTRACTID   = "@mozilla.org/process/ipc-buffer;1";
+const ENIG_IPCSERVICE_CONTRACTID  = "@mozilla.org/process/ipc-service;1";
 const ENIG_PIPEFILTERLISTENER_CONTRACTID = "@mozilla.org/process/pipe-filter-listener;1";
 const ENIG_ENIGMAIL_CONTRACTID    = "@mozdev.org/enigmail/enigmail;1";
 const ENIG_ENIGMIMELISTENER_CONTRACTID = "@mozilla.org/enigmail/mime-listener;1";
@@ -65,6 +67,9 @@ const ENIG_LOCALE_SVC_CONTRACTID = "@mozilla.org/intl/nslocaleservice;1";
 const ENIG_DATE_FORMAT_CONTRACTID = "@mozilla.org/intl/scriptabledateformat;1"
 const ENIG_ACCOUNT_MANAGER_CONTRACTID = "@mozilla.org/messenger/account-manager;1";
 const ENIG_XPCOM_APPINFO = "@mozilla.org/xre/app-info;1";
+const ENIG_THREAD_MANAGER_CID = "@mozilla.org/thread-manager;1";
+const ENIG_SIMPLEURI_CONTRACTID   = "@mozilla.org/network/simple-uri;1";
+const ENIG_SEAMONKEY_ID = "{92650c4d-4b8e-4d2a-b7eb-24ecf4f6b63a}"
 
 
 const ENIG_LOCALFILEOUTPUTSTREAM_CONTRACTID =
@@ -75,6 +80,8 @@ const ENIG_SAVEASCHARSET_CONTRACTID = "@mozilla.org/intl/saveascharset;1";
 
 const ENIG_STREAMCONVERTERSERVICE_CID_STR =
       "{892FFEB0-3F80-11d3-A16C-0050041CAF44}";
+const ENIG_EXTENSION_GUID = "{847b3a00-7ab1-11d4-8f02-006008948af5}";
+
 
 const ENIG_ISCRIPTABLEUNICODECONVERTER_CONTRACTID = "@mozilla.org/intl/scriptableunicodeconverter";
 
@@ -122,6 +129,10 @@ const ENIG_BUTTON_POS_1           = 1 << 8;
 const ENIG_BUTTON_POS_2           = 1 << 16;
 const ENIG_BUTTON_TITLE_IS_STRING = 127;
 
+const ENIG_HEADERMODE_KEYID = 0x01;
+const ENIG_HEADERMODE_URL   = 0x10;
+
+
 const ENIG_THREE_BUTTON_STRINGS   = (ENIG_BUTTON_TITLE_IS_STRING * ENIG_BUTTON_POS_0) +
                                (ENIG_BUTTON_TITLE_IS_STRING * ENIG_BUTTON_POS_1) +
                                (ENIG_BUTTON_TITLE_IS_STRING * ENIG_BUTTON_POS_2);
@@ -223,11 +234,21 @@ function GetEnigmailSvc() {
 
       if (firstInitialization) {
         // Display initialization error alert
-        var errMsg = gEnigmailSvc.initializationError ? gEnigmailSvc.initializationError : "Error in initializing Enigmail service";
+        var errMsg = gEnigmailSvc.initializationError ? gEnigmailSvc.initializationError : EnigGetString("accessError");
 
         errMsg += "\n\n"+EnigGetString("avoidInitErr");
 
-        EnigAlertPref("Enigmail: "+errMsg, "initAlert");
+
+        var checkedObj = {value: false};
+        if (EnigGetPref("initAlert")) {
+          var r = EnigLongAlert("Enigmail: "+errMsg, EnigGetString("dlgNoPrompt"), null, ":help", null, checkedObj);
+          if (r >= 0 && checkedObj.value) {
+            EnigSetPref("initAlert", false);
+          }
+          if (r == 1) {
+            EnigHelpWindow("initError");
+          }
+        }
         if (EnigGetPref("initAlert")) {
           gEnigmailSvc.initializationAttempted = false;
           gEnigmailSvc = null;
@@ -333,6 +354,28 @@ function EnigUpgradeHeadersView() {
   catch (ex) {}
 }
 
+function EnigUpgradeCustomHeaders() {
+  try {
+    var extraHdrs = " " + gEnigPrefRoot.getCharPref("mailnews.headers.extraExpandedHeaders").toLowerCase() + " ";
+
+    var extraHdrList = [
+      "x-enigmail-version",
+      "content-transfer-encoding",
+      "openpgp",
+      "x-mimeole",
+      "x-bugzilla-reason",
+      "x-php-bug" ];
+
+    for (hdr in extraHdrList) {
+      extraHdrs = extraHdrs.replace(" "+extraHdrList[hdr]+" ", " ");
+    }
+
+    extraHdrs = extraHdrs.replace(/^ */, "").replace(/ *$/, "");
+    gEnigPrefRoot.setCharPref("mailnews.headers.extraExpandedHeaders", extraHdrs)
+  }
+  catch(ex) {}
+}
+
 function EnigUpgradePgpMime() {
   var pgpMimeMode = false;
   try {
@@ -359,17 +402,23 @@ function EnigUpgradePgpMime() {
 
 function EnigConfigure() {
   var oldVer=EnigGetPref("configuredVersion");
-  if (oldVer == "") {
-    EnigOpenSetupWizard();
-  }
-  else if (oldVer < "0.95") {
-    try {
-      EnigUpgradeHeadersView();
-      EnigUpgradePgpMime();
-      EnigUpgradeRecipientsSelection();
+
+  try {
+    var vc = Components.classes["@mozilla.org/xpcom/version-comparator;1"].getService(Components.interfaces.nsIVersionComparator);
+    if (oldVer == "") {
+      EnigOpenSetupWizard();
     }
-    catch (ex) {}
+    else if (oldVer < "0.95") {
+      try {
+        EnigUpgradeHeadersView();
+        EnigUpgradePgpMime();
+        EnigUpgradeRecipientsSelection();
+      }
+      catch (ex) {}
+    }
+    else if (vc.compare(oldVer, "1.0") < 0) EnigUpgradeCustomHeaders();
   }
+  catch(ex) {};
   EnigSetPref("configuredVersion", gEnigmailVersion);
   EnigSavePrefs();
 }
@@ -544,9 +593,30 @@ function EnigAlert(mesg) {
   gEnigPromptSvc.alert(window, EnigGetString("enigAlert"), mesg);
 }
 
-function EnigLongAlert(mesg) {
-  window.openDialog("chrome://enigmail/content/enigmailAlertDlg.xul",
-            "", "chrome,modal,centerscreen", {msgtext: mesg});
+/**
+ * Displays an alert dialog with 3-4 optional buttons.
+ * prefName: if not null, display checkbox "don't show again", linked to the prefName preference
+ * button-Labels: use "&" to indicate access key
+ *     use "buttonType:label" or ":buttonType" to indicate special button types
+ *        (buttonType is one of cancel, help, extra1, extra2)
+ * return: 0-2: button Number pressed
+ *          -1: ESC or close window button pressed
+ *
+ */
+function EnigLongAlert(mesg, checkBoxLabel, okLabel, labelButton2, labelButton3, checkedObj) {
+  var result = {
+    value: -1,
+    checked: false
+  };
+  window.openDialog("chrome://enigmail/content/enigmailAlertDlg.xul", "",
+            "chrome,centerscreen,modal",
+            {msgtext: mesg, checkboxLabel: checkBoxLabel, button1: okLabel, button2: labelButton2, button3: labelButton3},
+            result);
+
+  if (checkBoxLabel) {
+    checkedObj.value=result.checked
+  }
+  return result.value;
 }
 
 function EnigAlertCount(countPrefName, mesg) {
@@ -587,49 +657,121 @@ function EnigAlertPref(mesg, prefText) {
   }
 }
 
-function EnigConfirm(mesg) {
+// Confirmation dialog with OK / Cancel buttons (both customizable)
+function EnigConfirm(mesg, okLabel, cancelLabel) {
   var dummy=new Object();
 
+  var buttonTitles = 0;
+  if (okLabel == null && cancelLabel == null) {
+    buttonTitles = (gEnigPromptSvc.BUTTON_TITLE_YES * ENIG_BUTTON_POS_0) +
+                   (gEnigPromptSvc.BUTTON_TITLE_NO * ENIG_BUTTON_POS_1);
+  }
+  else {
+    if (okLabel != null) {
+      buttonTitles += (gEnigPromptSvc.BUTTON_TITLE_IS_STRING * gEnigPromptSvc.BUTTON_POS_0);
+    }
+    else {
+      buttonTitles += gEnigPromptSvc.BUTTON_TITLE_OK * ENIG_BUTTON_POS_0;
+    }
+
+    if (cancelLabel != null) {
+      buttonTitles += (gEnigPromptSvc.BUTTON_TITLE_IS_STRING * gEnigPromptSvc.BUTTON_POS_1);
+    }
+    else {
+      buttonTitles += gEnigPromptSvc.BUTTON_TITLE_CANCEL * ENIG_BUTTON_POS_1;
+    }
+  }
+
   var buttonPressed = gEnigPromptSvc.confirmEx(window,
                         EnigGetString("enigConfirm"),
                         mesg,
-                        (gEnigPromptSvc.BUTTON_TITLE_YES * ENIG_BUTTON_POS_0) +
-                        (gEnigPromptSvc.BUTTON_TITLE_NO * ENIG_BUTTON_POS_1),
-                        null, null, null,
+                        buttonTitles,
+                        okLabel, cancelLabel, null,
                         null, dummy);
 
   return (buttonPressed == 0);
 }
 
-function EnigConfirmPref(mesg, prefText) {
+
+function EnigConfirmPref(mesg, prefText, okLabel, cancelLabel) {
   const notSet = 0;
   const yes = 1;
   const no = 2;
+  const display = true;
+  const dontDisplay = false;
+
+  var buttonTitles = 0;
+  if (okLabel == null && cancelLabel == null) {
+    buttonTitles = (gEnigPromptSvc.BUTTON_TITLE_YES * ENIG_BUTTON_POS_0) +
+                   (gEnigPromptSvc.BUTTON_TITLE_NO * ENIG_BUTTON_POS_1);
+  }
+  else {
+    if (okLabel != null) {
+      buttonTitles += (gEnigPromptSvc.BUTTON_TITLE_IS_STRING * gEnigPromptSvc.BUTTON_POS_0);
+    }
+    else {
+      buttonTitles += gEnigPromptSvc.BUTTON_TITLE_OK * ENIG_BUTTON_POS_0;
+    }
+
+    if (cancelLabel != null) {
+      buttonTitles += (gEnigPromptSvc.BUTTON_TITLE_IS_STRING * gEnigPromptSvc.BUTTON_POS_1);
+    }
+    else {
+      buttonTitles += gEnigPromptSvc.BUTTON_TITLE_CANCEL * ENIG_BUTTON_POS_1;
+    }
+  }
 
   var prefValue = EnigGetPref(prefText);
-  switch (prefValue) {
-  case notSet:
-    var checkBoxObj = { value: false} ;
-    var buttonPressed = gEnigPromptSvc.confirmEx(window,
-                          EnigGetString("enigConfirm"),
-                          mesg,
-                          (gEnigPromptSvc.BUTTON_TITLE_YES * ENIG_BUTTON_POS_0) +
-                          (gEnigPromptSvc.BUTTON_TITLE_NO * ENIG_BUTTON_POS_1),
-                          null, null, null,
-                          EnigGetString("dlgKeepSetting"), checkBoxObj);
-    if (checkBoxObj.value) {
-      EnigSetPref(prefText, (buttonPressed==0 ? yes : no));
+
+  if (typeof(prefValue) != "boolean") {
+    // number: remember user's choice
+    switch (prefValue) {
+    case notSet:
+      var checkBoxObj = { value: false} ;
+      var buttonPressed = gEnigPromptSvc.confirmEx(window,
+                            EnigGetString("enigConfirm"),
+                            mesg,
+                            buttonTitles,
+                            okLabel, cancelLabel, null,
+                            EnigGetString("dlgKeepSetting"), checkBoxObj);
+      if (checkBoxObj.value) {
+        EnigSetPref(prefText, (buttonPressed==0 ? yes : no));
+      }
+      return (buttonPressed==0 ? 1 : 0);
+
+    case yes:
+      return 1;
+
+    case no:
+      return 0;
+
+    default:
+      return -1;
     }
-    return (buttonPressed==0 ? 1 : 0);
+  }
+  else {
+    // boolean: "do not show this dialog anymore" (and return default)
+    switch (prefValue) {
+    case display:
+      var checkBoxObj = { value: false} ;
+      var buttonPressed = gEnigPromptSvc.confirmEx(window,
+                            EnigGetString("enigConfirm"),
+                            mesg,
+                            buttonTitles,
+                            okLabel, cancelLabel, null,
+                            EnigGetString("dlgNoPrompt"), checkBoxObj);
+      if (checkBoxObj.value) {
+        EnigSetPref(prefText, false);
+      }
+      return (buttonPressed==0 ? 1 : 0);
 
-  case yes:
-    return 1;
+    case dontDisplay:
+      return 1;
 
-  case no:
-    return 0;
+    default:
+      return -1;
+    }
 
-  default:
-    return -1;
   }
 }
 
@@ -666,6 +808,7 @@ function EnigOverrideAttribute(elementIdList, attrName, prefix, suffix) {
 function EnigPrefWindow(showBasic, clientType, selectTab) {
   DEBUG_LOG("enigmailCommon.js: EnigPrefWindow\n");
 
+  GetEnigmailSvc();
   if (showBasic && clientType == "seamonkey" && selectTab==null) {
     // Open the seamonkey pref window
     goPreferences("securityItem",
@@ -915,7 +1058,7 @@ function EnigConvertFromUnicode(text, charset) {
 
 
 function EnigConvertToUnicode(text, charset) {
-  DEBUG_LOG("enigmailCommon.js: EnigConvertToUnicode: "+charset+"\n");
+  // DEBUG_LOG("enigmailCommon.js: EnigConvertToUnicode: "+charset+"\n");
 
   if (!text || !charset /*|| (charset.toLowerCase() == "iso-8859-1")*/)
     return text;
@@ -928,7 +1071,7 @@ function EnigConvertToUnicode(text, charset) {
     return unicodeConv.ConvertToUnicode(text);
 
   } catch (ex) {
-    DEBUG_LOG("enigmailCommon.js: EnigConvertToUnicode: caught an exception\n");
+    DEBUG_LOG("enigmailCommon.js: EnigConvertToUnicode: caught an exception while converting'"+text+"' to "+charset+"\n");
     return text;
   }
 }
@@ -945,7 +1088,7 @@ function EnigConvertGpgToUnicode(text) {
         a=text.search(/\\x[0-9a-fA-F]{2}/);
     }
 
-    text = EnigConvertToUnicode(text, "utf-8");
+    text = EnigConvertToUnicode(text, "utf-8").replace(/\\e3A/g, ":");
   }
 
   return text;
@@ -1091,8 +1234,13 @@ function EnigClearPassphrase() {
   if (!enigmailSvc)
     return;
 
-  enigmailSvc.clearCachedPassphrase();
-  EnigAlertPref(EnigGetString("passphraseCleared"), "warnClearPassphrase");
+  if (enigmailSvc.useGpgAgent()) {
+    EnigAlert(EnigGetString("passphraseCannotBeCleared"))
+  }
+  else {
+    enigmailSvc.clearCachedPassphrase();
+    EnigAlertPref(EnigGetString("passphraseCleared"), "warnClearPassphrase");
+  }
 }
 
 function EnigOpenWin (winName, spec, winOptions, optList) {
@@ -1186,19 +1334,36 @@ function EnigViewDebugLog() {
 function EnigKeygen() {
   DEBUG_LOG("enigmailCommon.js: EnigKeygen\n");
 
-  window.openDialog('chrome://enigmail/content/enigmailKeygen.xul',
+  window.open('chrome://enigmail/content/enigmailKeygen.xul',
                     "enigmail:generateKey",
-                    'chrome,dialog,modal,resizable=yes,width=600');
+                    'chrome,dialog,modal,resizable=yes');
 
 }
 
 function EnigKeyManager() {
   DEBUG_LOG("enigmailCommon.js: EnigKeygen\n");
+  GetEnigmailSvc();
   EnigOpenWin("enigmail:KeyManager",
               "chrome://enigmail/content/enigmailKeyManager.xul",
               "resizable");
 }
 
+
+function EnigLaunchFile(fileName) {
+  try {
+    var mimeService = Components.classes[ENIG_MIME_CONTRACTID].getService(Components.interfaces.nsIMIMEService);
+    var fileMimeType = mimeService.getTypeFromFile(fileName);
+    var fileMimeInfo = mimeService.getFromTypeAndExtension(fileMimeType, fileExt);
+
+    fileMimeInfo.launchWithFile(fileName);
+  }
+  catch (ex) {
+    // if the attachment file type is unknown, an exception is thrown,
+    // so let it be handled by a browser window
+    enigLoadExternalURL(outFileUri.asciiSpec);
+  }
+}
+
 // retrieves the most recent navigator window (opens one if need be)
 function EnigLoadURLInNavigatorWindow(url, aOpenFlag)
 {
@@ -1323,6 +1488,12 @@ function EnigGetOS () {
 
 }
 
+function EnigGetVersion() {
+  return Components.classes["@mozilla.org/extensions/manager;1"].
+    getService(Components.interfaces.nsIExtensionManager).
+    getItemForID(ENIG_EXTENSION_GUID).version
+}
+
 function EnigDisplayPrefs(showDefault, showPrefs, setPrefs) {
   DEBUG_LOG("enigmailCommon.js: EnigDisplayPrefs\n");
 
@@ -1551,10 +1722,10 @@ function EnigObtainKeyList(secretOnly, refresh) {
 function EnigLoadKeyList(refresh, keyListObj) {
   DEBUG_LOG("enigmailCommon.js: EnigLoadKeyList\n");
 
+  const TRUSTLEVEL_SORTED="oidre-qnmfu"; // trust level sorted by increasing level of trust
+
   var sortUsers = function (a, b) {
-
-   if (a.userId.toLowerCase()<b.userId.toLowerCase()) { return -1;} else {return 1; }
-
+    if (a.userId.toLowerCase()<b.userId.toLowerCase()) { return -1; } else { return 1; }
   }
 
   var aGpgUserList = EnigObtainKeyList(false, refresh);
@@ -1562,7 +1733,7 @@ function EnigLoadKeyList(refresh, keyListObj) {
 
   var aGpgSecretsList = EnigObtainKeyList(true, refresh);
   if (!aGpgSecretsList && !refresh) {
-    if (EnigConfirm(EnigGetString("noSecretKeys"))) {
+    if (EnigConfirm(EnigGetString("noSecretKeys"), EnigGetString("keyMan.button.generateKey"), EnigGetString("keyMan.button.skip"))) {
       EnigKeygen();
       EnigLoadKeyList(true, keyListObj);
     }
@@ -1602,12 +1773,16 @@ function EnigLoadKeyList(refresh, keyListObj) {
           listRow[USER_ID] = "-";
         }
         if (typeof(keyObj.userId) != "string") {
-          keyObj.userId=EnigConvertGpgToUnicode(listRow[USER_ID]).replace(/\\e3A/g, ":");
+          keyObj.userId=EnigConvertGpgToUnicode(listRow[USER_ID]);
           keyListObj.keySortList.push({userId: keyObj.userId, keyId: keyObj.keyId});
+          if (TRUSTLEVEL_SORTED.indexOf(listRow[KEY_TRUST]) < TRUSTLEVEL_SORTED.indexOf(keyObj.keyTrust)) {
+            // reduce key trust if primary UID is less trusted than public key
+            keyObj.keyTrust = listRow[KEY_TRUST];
+          }
         }
         else {
           var subUserId = {
-            userId: EnigConvertGpgToUnicode(listRow[USER_ID]).replace(/\\e3A/g, ":"),
+            userId: EnigConvertGpgToUnicode(listRow[USER_ID]),
             keyTrust: listRow[KEY_TRUST],
             type: "uid"
           }
@@ -1643,13 +1818,75 @@ function EnigLoadKeyList(refresh, keyListObj) {
   keyListObj.keySortList.sort(sortUsers);
 }
 
-
-function EnigEditKeyTrust(userId, keyId) {
-  var inputObj = {
-    keyId: keyId,
-    userId: userId
+function EnigGetSecretKeys() {
+  // return a sorted array containing objects of (valid, usable) secret keys.
+  // @return: [ {name: <userId>, id: 0x1234ABCD, created: YYYY-MM-DD },  { ... } ]
+  var enigmailSvc = GetEnigmailSvc();
+  if (!enigmailSvc) {
+    EnigAlert(EnigGetString("accessError"));
+    window.close();
+    return null;
   }
-  window.openDialog("chrome://enigmail/content/enigmailEditKeyTrustDlg.xul","", "dialog,modal,centerscreen,resizable", inputObj);
+  var exitCodeObj = new Object();
+  var statusFlagsObj = new Object();
+  var errorMsgObj = new Object();
+  var keyList=enigmailSvc.getUserIdList(true, false, exitCodeObj, statusFlagsObj, errorMsgObj);
+
+  if (exitCodeObj.value != 0) {
+    EnigAlert(errorMsgObj.value);
+    window.close();
+    return null;
+  }
+
+  var userList=keyList.split(/\n/);
+  var secretKeyList = new Array();
+  var secretKeyCreated = new Array();
+  var i;
+  var keyId = null;
+
+  var keyId = null;
+  var keys = [];
+  for (i=0; i < userList.length; i++) {
+    if (userList[i].substr(0,4) == "sec:") {
+      var aLine=userList[i].split(/:/);
+      keyId = aLine[4];
+      secretKeyCreated[keyId] = EnigGetDateTime(aLine[5], true, false);
+      secretKeyList.push(keyId);
+    }
+  }
+
+  keyList = enigmailSvc.getKeyDetails(secretKeyList.join(" "), false);
+  userList=keyList.split(/\n/);
+
+  for (var i=0; i < userList.length; i++) {
+    var aLine = userList[i].split(/:/);
+    switch (aLine[0]) {
+    case "pub":
+      if (aLine[1] == "u") keyId = aLine[4]; // public key is ultimately trusted
+      break;
+    case "uid":
+      if ((keyId != null) && (aLine[1] == 'u')) {
+        // UID is valid and ultimately trusted
+        keys.push({ name: EnigConvertGpgToUnicode(aLine[9]),
+                    id: keyId,
+                    created: secretKeyCreated[keyId]});
+        keyId = null;
+      }
+    }
+  }
+
+  keys.sort(function(a,b) { return a.name == b.name ? (a.id < b.id ? -1 : 1) : (a.name.toLowerCase() < b.name.toLowerCase() ? -1 : 1); });
+  return keys;
+}
+
+function EnigEditKeyTrust(userIdArr, keyIdArr) {
+  var inputObj = {
+    keyId: keyIdArr,
+    userId: userIdArr
+  }
+  var resultObj = { refresh: false };
+  window.openDialog("chrome://enigmail/content/enigmailEditKeyTrustDlg.xul","", "dialog,modal,centerscreen,resizable", inputObj, resultObj);
+  return resultObj.refresh;
 }
 
 
@@ -1659,9 +1896,79 @@ function EnigSignKey(userId, keyId, signingKeyHint) {
     userId: userId,
     signingKeyHint: signingKeyHint
   }
-  window.openDialog("chrome://enigmail/content/enigmailSignKeyDlg.xul","", "dialog,modal,centerscreen,resizable", inputObj);
+  var resultObj = { refresh: false };
+  window.openDialog("chrome://enigmail/content/enigmailSignKeyDlg.xul","", "dialog,modal,centerscreen,resizable", inputObj, resultObj);
+  return resultObj.refresh;
 }
 
+
+function EnigChangeKeyPwd(keyId, userId) {
+  var inputObj = {
+    keyId: keyId,
+    userId: userId
+  };
+
+  var enigmailSvc = GetEnigmailSvc();
+  if (!enigmailSvc)
+    return;
+
+  if (! enigmailSvc.useGpgAgent()) {
+    window.openDialog("chrome://enigmail/content/enigmailChangePasswd.xul",
+        "", "dialog,modal,centerscreen", inputObj);
+  }
+  else {
+    // gpg-agent will handle everything
+    var errorMsgObj = new Object();
+    var r = enigmailSvc.simpleChangePassphrase(window, "0x"+keyId, errorMsgObj);
+
+    if (r != 0) {
+      EnigAlert(EnigGetString("changePassFailed")+"\n\n"+errorMsgObj.value);
+    }
+  }
+}
+
+
+function EnigRevokeKey(keyId, userId) {
+  var enigmailSvc = GetEnigmailSvc();
+  if (!enigmailSvc)
+    return false;
+
+  var userDesc="0x"+keyId.substr(-8,8)+" - "+userId;
+  if (!EnigConfirm(EnigGetString("revokeKeyAsk", userDesc), EnigGetString("keyMan.button.revokeKey"))) return;
+
+  var tmpDir=EnigGetTempDir();
+
+  try {
+    var revFile = Components.classes[ENIG_LOCAL_FILE_CONTRACTID].createInstance(Components.interfaces.nsILocalFile);
+    revFile.initWithPath(tmpDir);
+    if (!(revFile.isDirectory() && revFile.isWritable())) {
+      EnigAlert(EnigGetString("noTempDir"));
+      return false;
+    }
+  }
+  catch (ex) {}
+  revFile.append("revkey.asc");
+  revFile.createUnique(Components.interfaces.nsIFile.NORMAL_FILE_TYPE, 0600);
+
+  var errorMsgObj = {};
+  var r=enigmailSvc.genRevokeCert(window, "0x"+keyId, revFile.path, "0", "", errorMsgObj);
+  if (r != 0) {
+    revFile.remove(false);
+    EnigAlert(EnigGetString("revokeKeyFailed")+"\n\n"+errorMsgObj.value);
+    return false;
+  }
+  r = enigmailSvc.importKeyFromFile(window, revFile, errorMsgObj);
+  revFile.remove(false);
+  if (r != 0) {
+    EnigAlert(EnigGetString("revokeKeyFailed")+"\n\n"+EnigConvertGpgToUnicode(errorMsgObj.value));
+  }
+  else {
+    EnigAlert(EnigGetString("revokeKeyOk"));
+  }
+  return (r == 0);
+}
+
+
 function EnigShowPhoto(keyId, userId, photoNumber) {
   var enigmailSvc = GetEnigmailSvc();
   if (enigmailSvc) {
@@ -1697,6 +2004,13 @@ function EnigShowPhoto(keyId, userId, photoNumber) {
   }
 }
 
+function EnigGetFilePath (nsFileObj) {
+  if (EnigGetOS() == "WINNT") {
+    return EnigConvertToUnicode(nsFileObj.persistentDescriptor, "utf-8");
+  }
+  return EnigConvertFromUnicode(nsFileObj.path, "utf-8");
+}
+
 function EnigCreateRevokeCert(keyId, userId) {
   var defaultFileName = userId.replace(/[\<\>]/g, "");
   defaultFileName += " (0x"+keyId.substr(-8,8)+") rev.asc"
@@ -1711,7 +2025,7 @@ function EnigCreateRevokeCert(keyId, userId) {
     return -1;
 
   var errorMsgObj = {};
-  var r=enigmailSvc.genRevokeCert(window, "0x"+keyId, outFile.path, "1", "", errorMsgObj);
+  var r=enigmailSvc.genRevokeCert(window, "0x"+keyId, EnigGetFilePath(outFile), "1", "", errorMsgObj);
   if (r != 0) {
     EnigAlert(EnigGetString("revokeCertFailed")+"\n\n"+errorMsgObj.value);
   }
@@ -1759,6 +2073,9 @@ function EnigGetTrustLabel(trustCode) {
   case 'u':
     keyTrust=EnigGetString("keyTrust.ultimate");
     break;
+  case 'g':
+    keyTrust=EnigGetString("keyTrust.group");
+    break;
   case '-':
     keyTrust="-";
     break;
@@ -1842,4 +2159,25 @@ function EnigCollapseAdvanced(obj, attribute, dummy) {
 
     obj = obj.nextSibling;
   }
-}
\ No newline at end of file
+}
+
+
+function EnigOpenURL(event, hrefObj) {
+  var xulAppinfo = Components.classes["@mozilla.org/xre/app-info;1"].getService(Components.interfaces.nsIXULAppInfo);
+  if (xulAppinfo.ID == ENIG_SEAMONKEY_ID) return;
+
+  try {
+    var ioservice  = Components.classes["@mozilla.org/network/io-service;1"].
+                  getService(Components.interfaces.nsIIOService);
+    var iUri = ioservice.newURI(hrefObj.href, null, null);
+    var eps  = Components.classes["@mozilla.org/uriloader/external-protocol-service;1"].
+                  getService(Components.interfaces.nsIExternalProtocolService);
+
+    eps.loadURI(iUri, null);
+
+    event.preventDefault();
+    event.stopPropagation();
+  }
+  catch (ex) {}
+}
+
